@@ -1,8 +1,8 @@
    
-fit.model <- function(data, model.filename, restrictions.filename = NULL, n.optim = 5, fia = NULL, ci = 95, starting.values = NULL, lower.bound = 0, upper.bound = 1, output = c("standard", "fia", "full"), reparam.ineq = TRUE, fit.aggregated = TRUE, sort.param = TRUE, show.messages = TRUE, model.type = c("easy", "eqn", "eqn2"),  multicore = c("none", "individual", "n.optim", "fia"), sfInit = FALSE, nCPU = 2, control = list(), use.gradient = TRUE, use.hessian = FALSE, check.model = TRUE){
+fit.model <- function(data, model.filename, restrictions.filename = NULL, n.optim = 5, fia = NULL, ci = 95, starting.values = NULL, lower.bound = 0, upper.bound = 1, output = c("standard", "fia", "full"), reparam.ineq = TRUE, fit.aggregated = TRUE, sort.param = TRUE, show.messages = TRUE, model.type = c("easy", "eqn", "eqn2"),  multicore = c("none", "individual", "n.optim", "fia"), sfInit = FALSE, nCPU = 2, control = list(), use.gradient = TRUE, use.hessian = FALSE, check.model = TRUE, args.fia = list()){
 	
 	llk.model <- function(Q, unlist.model, data, param.names, n.params, lower.bound, upper.bound, llk.gradient, llk.hessian, tmp.env, ...){
-		if (check.model) {
+		try({
             if (length(upper.bound) == 1) {
                 Q[Q > upper.bound] <- upper.bound
             } else {
@@ -13,12 +13,12 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
             } else {
                 Q[Q < lower.bound] <- lower.bound[Q < lower.bound]
             }
-        }
+        }, silent = TRUE)
 		#tmpllk.env <- new.env()
 		for (i in seq_len(n.params))  assign(param.names[i],Q[i], envir = tmp.env)
 		
 		model.eval <- vapply(unlist.model, eval, envir = tmp.env, 0)
-		if (check.model) if (any(model.eval < 0)) stop(paste("Model not constructed well. Line ", which(model.eval < 0), " produces probabilities < 0!", sep = ""))
+		if (check.model) if (isTRUE(any(model.eval < 0))) stop(paste("Model not constructed well. Line ", which(model.eval < 0), " produces probabilities < 0!", sep = ""))
 		llk <- data * log(model.eval)
 		llk[data == 0] <- 0
 		llk <- sum(llk)
@@ -34,7 +34,7 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
 	}
 	
 	llk.gradient.funct <- function(Q, unlist.model, data, param.names, n.params, lower.bound, upper.bound, llk.gradient, llk.hessian, tmp.env, ...){
-		if (check.model) {
+		try( {
             if (length(upper.bound) == 1) {
                 Q[Q > upper.bound] <- upper.bound
             } else {
@@ -45,7 +45,7 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
             } else {
                 Q[Q < lower.bound] <- lower.bound[Q < lower.bound]
             }
-        }
+        }, silent = TRUE)
 		#tmpllk.env <- new.env()
 		for (i in 1:n.params)  assign(param.names[i],Q[i], envir = tmp.env)
 		
@@ -56,7 +56,7 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
 	}
 
 	llk.hessian.funct <- function(Q, unlist.model, data, param.names, n.params, lower.bound, upper.bound, llk.gradient, llk.hessian, tmp.env, ...){
-		if (check.model) {
+		try( {
             if (length(upper.bound) == 1) {
                 Q[Q > upper.bound] <- upper.bound
             } else {
@@ -67,7 +67,7 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
             } else {
                 Q[Q < lower.bound] <- lower.bound[Q < lower.bound]
             }
-        }
+        }, silent = TRUE)
 		#tmpllk.env <- new.env()
 		for (i in 1:n.params)  assign(param.names[i],Q[i], envir = tmp.env)
 		
@@ -152,15 +152,32 @@ fit.model <- function(data, model.filename, restrictions.filename = NULL, n.opti
 	llk.hessian <- tryCatch(.make.llk.hessian(llk.function, param.names, length.param.names), error = function(e) {message("hessian function cannot be build (probably derivation failure, see ?D"); NULL})
 	
 	if (!is.null(fia)) {
-		if (multiFit) {
-			data.new <- rbind(data, apply(data,2,sum))
-			fia.tmp <- get.mpt.fia(data.new, model.filename, restrictions.filename, fia, model.type, multicore = if (multicore[1] != "none") TRUE else FALSE)
-			fia.df <- fia.tmp[-dim(fia.tmp)[1],]
-			fia.agg.tmp <- fia.tmp[dim(fia.tmp)[1],]
-			fia.df <- list(fia.df, fia.agg.tmp)
-		} else {
-			fia.df <- get.mpt.fia(data, model.filename, restrictions.filename, fia, model.type, multicore = if (multicore[1] != "none") TRUE else FALSE)
-		}
+	  if (length(args.fia)) {
+	    nms <- names(args.fia)
+	    if (!is.list(args.fia) || is.null(args.fia)) stop("'args.fia' argument must be a named list")
+	    if (!all(nms %in% names(formals(get.mpt.fia)))) warning(paste0("Unrecognized arguments in 'args.fia' ignored: ", paste(args.fia[!nms %in% names(formals(get.mpt.fia))], collapes = " ")))
+	    args.fia <- args.fia[nms %in% names(formals(get.mpt.fia))]
+	    fia.args <- c(model.filename = model.filename, restrictions.filename = if (is.list(restrictions.filename)) list(restrictions.filename) else restrictions.filename, Sample = fia, model.type = list(model.type), multicore = if (multicore[1] != "none") TRUE else FALSE, args.fia)
+	  }
+	  else {
+	    fia.args <- list(model.filename = model.filename, restrictions.filename = restrictions.filename, Sample = fia, model.type = model.type, multicore = if (multicore[1] != "none") TRUE else FALSE)
+	  }
+	  args.fia <- NULL
+	  if (multiFit) {
+	    data.new <- rbind(data, apply(data,2,sum))
+	    fia.tmp <- tryCatch(do.call(get.mpt.fia, args = c(data = list(data.new), fia.args)), error = function(e) NULL)
+	    if (!is.null(fia.tmp)) {
+	      fia.df <- fia.tmp[-dim(fia.tmp)[1],]
+	      fia.agg.tmp <- fia.tmp[dim(fia.tmp)[1],]
+	      fia.df <- list(fia.df, fia.agg.tmp)
+	    } else fia.df <- NULL
+	  } else {
+	    fia.df <- tryCatch(do.call(get.mpt.fia, args = c(data = list(data.new), fia.args)), error = function(e) NULL)
+	  }
+	  if (is.null(fia.df)) {
+	    warning("Calculation of FIA failed. Model does not seem to be a BMPT!")
+	    fia <- NULL
+	  }
 	}
 	
 	# call the workhorse:	
